// Copyright (C) 2014 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function x=distfun_vectorrnd(varargin)
    // Random vectors. 
    //
    // Calling Sequence
    //   x=distfun_vectorrnd(r)
    //   x=distfun_vectorrnd(r,n)
    //   x=distfun_vectorrnd(r,n,sigma)
    //
    // Parameters
    //   r : a list with d items describing the multivariate random variable (see below)
    //   n : a 1-by-1 matrix of floating point integers, the number of rows in x (default n = 1)
    //   sigma : a d-by-d matrix of doubles, the covariance matrix, symmetric, positive definite (default = identity)
    //   x: a m-by-n matrix of doubles, the random numbers.
    //
    // Description
    // Generate a multivariate random vector given 
    // marginal and covariance matrix. 
    //
    // The covariance matrix sigma must have unit diagonal. 
    // Its non-diagonal terms must be in the range [0,1).
    //
    // If sigma is not provided (default), then 
    // the multivariate sample is generated by direct 
    // simulation, considering that the dimensions 
    // are independent. 
    // 
    // If sigma is provided, the algorithm produces a sample with 
    // Gaussian copula.
    //
    // The variable r is a list of n items, where r(i) is a list 
    // which describes the i-th random variable. 
    // The first item in r(i) is a string, which is the short name of the 
    // distribution. 
    // The used map is defined by distfun_list.
    //
    // Implementation notes
    //
    // If sigma is provided, the algorithm is the following.
    // <itemizedlist>
    //   <listitem>
    //     <para>
    // Generate Z=[Z1,...,Zd] with zero mean, and given covariance.
    //     </para>
    //   </listitem>
    //   <listitem>
    //     <para>
    // Compute U = [Phi(Z1),...,Phi(Zd)], 
    // where Phi is the standard normal cumulative distribution function.
    //     </para>
    //   </listitem>
    //   <listitem>
    //     <para>
    // Compute X = [Q1(U1),...,Qd(Ud)], 
    // where Qi is the quantile function of the i-th marginal.
    //     </para>
    //   </listitem>
    // </itemizedlist>
    // 
    // Examples
    // sigma = [
    // 3.0  0.5
    // 0.5  1.0
    // ]
    // v1=list("unif",5,12) // Uniform(min=5,max=12)
    // v2=list("norm",-5,7) // Normal(mean=-5,sigma=7)
    // r=list(v1,v2)
    // n=10 // Number of outcomes
    // x=distfun_vectorrnd(r,sigma,n)
    //
    // // Graphical "test" (requires Stixbox)
    // x=distfun_vectorrnd(r,sigma,1000);
    // scf();
    // for i=1:4
    //     subplot(2,2,i)
    //     rho=(i-1)/4;
    //     sigma = [1.0  rho;rho  1.0];
    //     x=distfun_vectorrnd(r,1000,sigma);
    //     plot(x(:,1),x(:,2),"bo");
    //     title("rho="+string(rho));
    // end
    // scf();
    // subplot(1,2,1)
    // histo(x(:,1));
    // title("Uniform(5,12)");
    // subplot(1,2,2)
    // histo(x(:,2));
    // title("Normal(-5,7)");
    //
    // Bibliography
    // http://web.ics.purdue.edu/~hwan/IE680/Lectures/Chap08Slides.pdf
    // http://fr.mathworks.com/help/stats/examples/simulating-dependent-random-variables-using-copulas.html
    //
    // Authors
    //   Copyright (C) 2014 - Michael Baudin

    function x = distfun_vectindep(r,n,d)
        // Generate with Independent copula
        // d : number of variables

        x=zeros(n,d)
        for i=1:d
            rvi=r(i) // i-th random variable
            shortname=rvi(1)
            rnd=distfun_name2func("rnd",shortname)
            x(:,i)=rnd(rvi(2:$),n,1)
        end
    endfunction

    function x = distfun_vectgaussian(r,n,d,sigma)
        // Generate with Gaussian copula
        // d : number of variables
        // 1. Generate Z=[Z1,...,Zd]
        d=size(sigma,"r")
        mu=zeros(1,d)
        z=distfun_mvnrnd(mu,sigma,n)
        // 2. Compute U = [Phi(Z1),...,Phi(Zd)]
        u=zeros(z)
        for i=1:d
            u(:,i)=distfun_normcdf(z(:,i),0,1)
        end
        // 3. Compute X = [Q1(U1),...,Qd(Ud)]
        previousprot = funcprot()
        x=zeros(u)
        for i=1:d
            rvi=r(i) // i-th random variable
            shortname=rvi(1)
            qua=distfun_name2func("inv",shortname)
            x(:,i)=qua(u(:,i),rvi(2:$))
        end
        funcprot(0)
    endfunction

    // Load Internals lib
    path = distfun_getpath (  )
    internallib  = lib(fullfile(path,"macros","internals"))

    [lhs,rhs]=argn()
    apifun_checkrhs ( "distfun_vectorrnd" , rhs , 1:3 )
    apifun_checklhs ( "distfun_vectorrnd" , lhs , 0:1 )
    //
    r = varargin(1)
    n = apifun_argindefault (varargin,2,1)
    sigma = apifun_argindefault (varargin,3,[])
    //
    // Check type
    apifun_checktype ( "distfun_vectorrnd" , r , "r" , 1 , "list" )
    apifun_checktype ( "distfun_vectorrnd" , n , "n" , 2 , "constant" )
    if (sigma<>[]) then
        apifun_checktype ( "distfun_vectorrnd" , sigma , "sigma" , 3 , "constant" )
    end
    //
    // Check size
    d=length(r)
    apifun_checkscalar("distfun_vectorrnd" , n , "n" , 2 )
    if (sigma<>[]) then
        apifun_checksquare("distfun_vectorrnd" , sigma , "sigma" , 3 )
        apifun_checkdims("distfun_vectorrnd" , sigma , "sigma" , 3 ,[d,d])
    end
    //
    // Check content
    allnames=distfun_list("shortnames")
    for i=1:d
        rvi=r(i) // i-th random variable
        apifun_checktype ( "distfun_vectorrnd" , rvi , "r(i)" , 1 , "list" )
        shortname=rvi(1)
        apifun_checktype ( "distfun_vectorrnd" , shortname , "r(i)(1)" , 1 , "string" )
        // Check that shortname is known
        if (~or(shortname==allnames)) then
            errmsg = msprintf(gettext("%s: Unknown distribution ""%s"" in r(%d)."), "distfun_vectorrnd" , shortname , i);
            error(errmsg)
        end
    end
    apifun_checkgreq("distfun_vectorrnd" , n , "n" , 2 , 1 )
    // Check sigma
    if (sigma<>[]) then
        apifun_checkgreq("distfun_vectorrnd" , sigma , "sigma" , 3 , 0. )
        apifun_checkloweq("distfun_vectorrnd" , sigma , "sigma" , 3 , 1. )
        // Check that diagonal is unity
        if (~and(diag(sigma)==ones(d,1))) then
            errmsg = msprintf(gettext("%s: Expected unit diagonal in argument ""sigma"" at input #%d."), "distfun_vectorrnd" , 3);
            error(errmsg)
        end
        // Check that non-diagonal is <1
        if (~and(sigma-eye(d,d)<1)) then
            errmsg = msprintf(gettext("%s: Expected <1 entries in argument ""sigma"" at input #%d."), "distfun_vectorrnd" , 3);
            error(errmsg)
        end
    end
    //
    // Proceed
    if (sigma==[]) then
        x = distfun_vectindep(r,n,d)
    else
        x = distfun_vectgaussian(r,n,d,sigma)
    end
endfunction
